/**
 * Copyright (c) 2015-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * @flow
 */
'use strict';

var setupBackend = require('../backend/backend');

module.exports = function (hook, agent) {
  var subs = [hook.sub('renderer-attached', function (_ref) {
    var id = _ref.id;
    var renderer = _ref.renderer;
    var helpers = _ref.helpers;

    agent.setReactInternals(id, helpers);
    helpers.walkTree(agent.onMounted.bind(agent, id), agent.addRoot.bind(agent, id));
  }), hook.sub('root', function (_ref2) {
    var renderer = _ref2.renderer;
    var element = _ref2.element;
    return agent.addRoot(renderer, element);
  }), hook.sub('mount', function (_ref3) {
    var renderer = _ref3.renderer;
    var element = _ref3.element;
    var data = _ref3.data;
    return agent.onMounted(renderer, element, data);
  }), hook.sub('update', function (_ref4) {
    var renderer = _ref4.renderer;
    var element = _ref4.element;
    var data = _ref4.data;
    return agent.onUpdated(element, data);
  }), hook.sub('unmount', function (_ref5) {
    var renderer = _ref5.renderer;
    var element = _ref5.element;
    return agent.onUnmounted(element);
  })];

  var success = setupBackend(hook);
  if (!success) {
    return;
  }

  hook.emit('react-devtools', agent);
  hook.reactDevtoolsAgent = agent;
  agent.on('shutdown', function () {
    subs.forEach(function (fn) {
      return fn();
    });
    hook.reactDevtoolsAgent = null;
  });
};