/**
 * Copyright (c) 2015-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * @flow
 */
'use strict';

var sep = '\u001f'; // separator
// var MAX_DEPTH = 100;

function crawlChildren(ptype, children, nodes, depth, graph) {
  var descendents = [];
  var keepCrawling = true; //depth < MAX_DEPTH;
  children.forEach(function (cid) {
    var child = nodes.get(cid);
    var isCustom = child.get('nodeType') === 'Composite';
    if (isCustom) {
      var name = child.get('name');
      if (!graph.nodes[name]) {
        graph.nodes[name] = 1;
      } else {
        graph.nodes[name] += 1;
      }
      var key = ptype + sep + name;
      if (graph.edges[key]) {
        graph.edges[key] += 1;
      } else {
        graph.edges[key] = 1;
      }
    }
    if (keepCrawling) {
      var grandChildren = child.get('children');
      if (grandChildren && Array.isArray(grandChildren)) {
        if (isCustom) {
          crawlChildren(name, grandChildren, nodes, depth + 1, graph);
        } else {
          descendents = descendents.concat(grandChildren);
        }
      }
    }
  });

  if (keepCrawling && descendents.length) {
    crawlChildren(ptype, descendents, nodes, depth + 1, graph);
  }
}

module.exports = crawlChildren;