/**
 * Copyright (c) 2015-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * @flow
 */
'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var globalHook = require('../../../backend/GlobalHook');
globalHook(window);

var Panel = require('../../../frontend/Panel');
var React = require('react');

var node = document.getElementById('container');
node.innerHTML = '<h1>Connecting to devtools</h1>';
var port = {};

function reload() {
  React.unmountComponentAtNode(node);
  node.innerHTML = '';
  React.render(React.createElement(Panel, _extends({ alreadyFoundReact: true }, config)), node);
}

window.addEventListener('message', function (event) {
  port = event.ports[0];
  var metaPort = event.ports[1];
  metaPort.onmessage = function (evt) {
    if (evt.data === 'show') {
      reload();
    } else if (evt.data === 'unload') {
      node.innerHTML = '<h1 id="message">Looking for React</h1>';
    } else if (evt.data.type === 'hasReact') {
      if (evt.data.val) {
        React.render(React.createElement(Panel, _extends({ alreadyFoundReact: true }, config)), node);
      } else {
        node.innerHTML = '<h1 id="message">No react found on page...</h1>';
      }
    }
  };
});

var config = {
  reload: reload,
  // checkForReact,
  inject: function inject(done) {
    var disconnected = false;

    var wall = {
      listen: function listen(fn) {
        port.onmessage = function (evt) {
          fn(evt.data);
        };
      },
      send: function send(data) {
        if (disconnected) {
          return;
        }
        port.postMessage(data);
      } };
    done(wall);
  } };